@@ -43,11 +43,14 @@ import socket
 import resource
 import traceback
 import multiprocessing
+from types import TracebackType  # noqa: F401
+from typing import Dict, List, Optional, Tuple, Type  # noqa: F401
 
 from tornado import process
 import notifier
 import notifier.signals as signals
 from OpenSSL import SSL
+from OpenSSL.crypto import X509  # noqa: F401
 
 from univention.lib.i18n import Translation
 
@@ -66,32 +69,36 @@ class MagicBucket(object):
 
 	'''Manages a connection (session) to the UMC server. Therefore it
 	ensures that without successful authentication no other command is
-	accepted. All commands are passed to the SessionHandler. After the user
+	accepted. All commands are passed to the :class:`~SessionHandler`. After the user
 	has authenticated the commands are passed on to the Processor.'''
 
 	def __init__(self):
-		self.__states = {}
+		# type: () -> None
+		self.__states = {}  # type: Dict[socket.socket, State]
 
-	def new(self, client, socket):
+	def new(self, client, sock):
+		# type: (str, socket.socket) -> None
 		"""Is called by the Server object to announce a new incoming
 		connection.
 
 		:param str client: IP address + port
-		:param fd socket: a file descriptor or socket object
+		:param socket.socket sock: a socket object
 		"""
 		CORE.info('Established connection: %s' % client)
-		state = State(client, socket)
+		state = State(client, sock)
 		state.session.signal_connect('success', notifier.Callback(self._response, state))
-		self.__states[socket] = state
-		notifier.socket_add(socket, self._receive)
+		self.__states[sock] = state
+		notifier.socket_add(sock, self._receive)
 		self.reset_connection_timeout(state)
 
 	def reset_connection_timeout(self, state):
+		# type: (State) -> None
 		state.reset_connection_timeout()
 		notifier.timer_remove(state._timer)
 		state._timer = notifier.timer_add(state.timeout * 1000, notifier.Callback(self._timed_out, state))
 
 	def _timed_out(self, state):
+		# type: (State) -> bool
 		"""Closes the connection after a specified timeout"""
 		if not state.active:
 			CORE.info('Session %r timed out' % (state,))
@@ -102,20 +109,22 @@ class MagicBucket(object):
 		return False
 
 	def exit(self):
+		# type: () -> None
 		'''Closes all open connections.'''
 		# remove all sockets
-		for sock in list(self.__states.keys()):
+		for sock in list(self.__states):
 			CORE.info('Shutting down connection %s' % sock)
 			self._cleanup(sock)
 
 	def _receive(self, sock):
+		# type: (socket.socket) -> bool
 		"""Signal callback: Handles incoming data. Processes SSL events
 		and parses the incoming data. If a valid UMCP was found it is
 		passed to _handle.
 
-		:param fd socket: file descriptor or socket object that reported incoming data
+		:param socket.socket sock: socket object that reported incoming data
 		"""
-		data = ''
+		data = b''
 
 		try:
 			data = sock.recv(RECV_BUFFER_SIZE)
@@ -168,6 +177,7 @@ class MagicBucket(object):
 		return True
 
 	def _do_send(self, sock):
+		# type: (socket.socket) -> bool
 		try:
 			state = self.__states[sock]
 		except KeyError:
@@ -198,9 +208,10 @@ class MagicBucket(object):
 			self._cleanup(sock)
 			return False
 
-		return (len(state.resend_queue) > 0)
+		return bool(state.resend_queue)
 
 	def _response(self, msg, state):
+		# type: (Message, State) -> None
 		''' Send UMCP response to client. If the status code is 250 the
 		module process is asking for exit. This method forfills the
 		request.'''
@@ -242,16 +253,17 @@ class MagicBucket(object):
 			CORE.error('FATAL ERROR: %s' % (traceback.format_exc(),))
 			self._cleanup(state.socket)
 
-	def _cleanup(self, socket):
-		state = self.__states.pop(socket, None)
+	def _cleanup(self, sock):
+		# type: (socket.socket) -> None
+		state = self.__states.pop(sock, None)
 		if state is None:
 			return
 
 		state.session.close_session()
 
-		notifier.socket_remove(socket)
+		notifier.socket_remove(sock)
 		try:
-			socket.close()
+			sock.close()
 		except Exception:
 			pass
 
@@ -270,9 +282,11 @@ class Server(signals.Provider):
 	:param bool magic: if an external session handler should be used
 	:param class magicClass: a reference to the class for the external session handler
 	:param bool load_ressources: if the modules and categories definitions should be loaded
+	:param int processes: Enable multi-process mode.
 	"""
 
 	def __init__(self, port=6670, ssl=True, unix=None, magic=True, magicClass=MagicBucket, load_ressources=True, processes=1):
+		# type: (int, bool, Optional[str], bool, Type[MagicBucket], bool, int) -> None
 		'''Initializes the socket to listen for requests'''
 		signals.Provider.__init__(self)
 
@@ -283,17 +297,19 @@ class Server(signals.Provider):
 
 		self.__port = port
 		self.__unix = unix
-		self.__realtcpsocket = None
-		self.__realunixsocket = None
+		self.__realtcpsocket = None  # type: Optional[socket.socket]
+		self.__realunixsocket = None  # type: Optional[socket.socket]
 		self.__ssl = ssl
 		self.__processes = processes
-		self._child_number = None
-		self._children = {}
+		self._child_number = None  # type: Optional[int]
+		self._children = {}  # type: Dict[int, int]
 		self.__magic = magic
 		self.__magicClass = magicClass
-		self.__bucket = None
+		self.__bucket = None  # type: Optional[MagicBucket]
+		self.crypto_context = None  # type: Optional[SSL.Context]
 
 	def __enter__(self):
+		# type: () -> Server
 		CORE.info('Initialising server process')
 		if self.__unix:
 			CORE.info('Using a UNIX socket')
@@ -311,19 +327,19 @@ class Server(signals.Provider):
 				continue
 			sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
 			sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-			sock.setblocking(0)
+			sock.setblocking(False)
 			fd = sock.fileno()
 			flags = fcntl.fcntl(fd, fcntl.F_GETFD)
 			fcntl.fcntl(fd, fcntl.F_SETFD, flags | fcntl.FD_CLOEXEC)
 
-		if self.__ssl and self.__port:
+		if self.__ssl and self.__realtcpsocket is not None:
 			CORE.info('Setting up SSL configuration')
 			self.crypto_context = SSL.Context(SSL.TLSv1_METHOD)
 			self.crypto_context.set_cipher_list(ucr.get('umc/server/ssl/ciphers', 'DEFAULT'))
 			self.crypto_context.set_options(SSL.OP_NO_SSLv2)
 			self.crypto_context.set_options(SSL.OP_NO_SSLv3)
 			self.crypto_context.set_verify(SSL.VERIFY_PEER, self.__verify_cert_cb)
-			dir = '/etc/univention/ssl/%s.%s' % (ucr['hostname'], ucr['domainname'])
+			dir = '/etc/univention/ssl/%(hostname)s.%(dirname)s' % ucr
 			try:
 				self.crypto_context.use_privatekey_file(os.path.join(dir, 'private.key'))
 				self.crypto_context.use_certificate_file(os.path.join(dir, 'cert.pem'))
@@ -345,13 +361,13 @@ class Server(signals.Provider):
 				self.connection.set_accept_state()
 				CRYPT.info('Server listening to SSL connections')
 				self.connection.listen(SERVER_MAX_CONNECTIONS)
-		elif not self.__ssl and self.__port:
+		elif not self.__ssl and self.__realtcpsocket is not None:
 			self.crypto_context = None
 			self.__realtcpsocket.bind(('', self.__port))
 			CRYPT.info('Server listening to TCP connections')
 			self.__realtcpsocket.listen(SERVER_MAX_CONNECTIONS)
 
-		if self.__unix:
+		if self.__unix and self.__realunixsocket is not None:
 			# ensure that the UNIX socket is only accessible by root
 			old_umask = os.umask(0o077)
 			try:
@@ -383,7 +399,7 @@ class Server(signals.Provider):
 
 		if self.__ssl:
 			notifier.socket_add(self.connection, self._connection)
-		if (not self.__ssl and self.__port):
+		elif self.__port:
 			notifier.socket_add(self.__realtcpsocket, self._connection)
 		if self.__unix:
 			notifier.socket_add(self.__realunixsocket, self._connection)
@@ -391,15 +407,17 @@ class Server(signals.Provider):
 		return self
 
 	def __verify_cert_cb(self, conn, cert, errnum, depth, ok):
+		# type: (SSL.Connection, X509, int, int, int) -> bool
 		CORE.info('__verify_cert_cb: Got certificate: %s' % cert.get_subject())
 		CORE.info('__verify_cert_cb: Got certificate issuer: %s' % cert.get_issuer())
 		CORE.info('__verify_cert_cb: errnum=%d depth=%d ok=%d' % (errnum, depth, ok))
-		return ok
+		return ok  # FIXME: should return true if verification passes and false otherwise.
 
-	def _connection(self, socket):
+	def _connection(self, sock):
+		# type: (socket.socket) -> bool
 		'''Signal callback: Invoked on incoming connections.'''
 		try:
-			socket, addr = socket.accept()
+			sock, addr = sock.accept()
 		except EnvironmentError as exc:
 			if exc.errno == errno.EAGAIN:
 				# got an EAGAIN --> try again later
@@ -415,8 +433,8 @@ class Server(signals.Provider):
 				soft, hard = resource.getrlimit(resource.RLIMIT_NOFILE)
 				resource.setrlimit(resource.RLIMIT_NOFILE, (soft + 2, hard + 2))
 				try:
-					socket, addr = socket.accept()
-					socket.close()
+					sock, addr = sock.accept()
+					sock.close()
 				except EnvironmentError:
 					pass
 				finally:
@@ -425,22 +443,23 @@ class Server(signals.Provider):
 				# unknown errno - log traceback and continue
 				CORE.error(traceback.format_exc())
 			return True
-		fd = socket.fileno()
+		fd = sock.fileno()
 		flags = fcntl.fcntl(fd, fcntl.F_GETFD)
 		fcntl.fcntl(fd, fcntl.F_SETFD, flags | fcntl.FD_CLOEXEC)
-		socket.setblocking(0)
+		sock.setblocking(False)
 		if addr:
 			client = '%s:%d' % (addr[0], addr[1])
 		else:
 			client = ''
 		CORE.info('Incoming connection from %s' % client)
-		if self.__magic:
-			self.__bucket.new(client, socket)
+		if self.__bucket is not None:
+			self.__bucket.new(client, sock)
 		else:
-			self.signal_emit('session_new', client, socket)
+			self.signal_emit('session_new', client, sock)
 		return True
 
 	def exit(self):
+		# type: () -> None
 		'''Shuts down all open connections.'''
 		CORE.warn('Shutting down all open connections')
 
@@ -469,10 +488,12 @@ class Server(signals.Provider):
 		self.__bucket = None
 
 	def __exit__(self, etype, exc, etraceback):
+		# type: (Optional[Type[BaseException]], Optional[BaseException], Optional[TracebackType]) -> None
 		self.exit()
 
 	@staticmethod
 	def reload():
+		# type: () -> None
 		"""Reloads resources like module and category definitions"""
 		CORE.info('Reloading resources: modules, categories')
 		moduleManager.load()
@@ -482,6 +503,7 @@ class Server(signals.Provider):
 
 	@staticmethod
 	def analyse_memory():
+		# type: () -> None
 		"""Print the number of living UMC objects. Helpful when analysing memory leaks."""
 		components = (
 			'protocol.server.State', 'protocol.session.ModuleProcess',
@@ -514,27 +536,31 @@ class State(object):
 	"""Holds information about the state of an active session
 
 	:param str client: IP address + port
-	:param fd socket: file descriptor or socket object
+	:param socket.socket sock: socket object
 	"""
 
 	__slots__ = ('client', 'socket', 'buffer', 'requests', 'resend_queue', 'session', 'timeout', '_timer')
 
-	def __init__(self, client, socket):
+	def __init__(self, client, sock):
+		# type: (str, socket.socket) -> None
 		self.client = client
-		self.socket = socket
+		self.socket = sock
 		self.buffer = b''
-		self.requests = {}
-		self.resend_queue = []
+		self.requests = {}  # type: Dict
+		self.resend_queue = []  # type: List[Tuple[str, bytes]]
 		self.session = SessionHandler()
 		self._timer = None
 		self.reset_connection_timeout()
 
 	def reset_connection_timeout(self):
+		# type: () -> None
 		self.timeout = SERVER_CONNECTION_TIMEOUT
 
 	@property
 	def active(self):
+		# type: () -> bool
 		return bool(self.requests or self.session.has_active_module_processes())
 
 	def __repr__(self):
+		# type: () -> str
 		return '<State(%s %r buffer=%d requests=%d processes=%s)>' % (self.client, self.socket, len(self.buffer), len(self.requests), self.session.has_active_module_processes())
