@@ -73,7 +73,7 @@ from ..modules.sanitizers import StringSanitizer, DictSanitizer
 from ..modules.decorators import sanitize, sanitize_args, simple_response, allow_get_request
 
 try:
-	from typing import Any, Dict, Iterable, List, Optional  # noqa F401
+	from typing import Any, Dict, Iterable, List, Optional  # noqa: F401
 except ImportError:
 	pass
 
@@ -664,7 +664,7 @@ class ProcessorBase(Base):
 		is shut down to save resources. The timer ticks each seconds to
 		handle glitches of the system clock.
 
-		:param Module module: a module
+		:param Module module: a UMC module
 		"""
 		if module._inactivity_timer is None:
 			module._inactivity_timer = notifier.timer_add(1000, notifier.Callback(self._inactivitiy_tick, module))
